function res = class_CVdemo2(opts,pars)
% class_CVdemo2: demonstrates cross-validated classification with SFA
%
% see [Berkes05] Pietro Berkes: Pattern recognition with Slow Feature
% Analysis. Cognitive Sciences EPrint Archive (CogPrint) 4104, http://cogprints.org/4104/ (2005)
%
% Instead of handwritten digits we use: 
% - with pars.GENERATE_P==2 or ==3 the Vowel benchmark dataset from UCI repository
% http://archive.ics.uci.edu/ml/datasets/Connectionist+Bench+%28Vowel+Recognition+-+Deterding+Data%29 
% - or with pars.GENERATE_P==4 the Breast Cancer dataset from UCI repository
% http://archive.ics.uci.edu/ml/datasets/Breast+Cancer+Wisconsin+%28Diagnostic%29
% - or with pars.GENERATE_P==9 the gesture data from Polak/Hein (FH Cologne).
% - or with pars.GENERATE_P==10 the new gesture data KH_I from Kristine Hein (FH Cologne).
% - or with pars.GENERATE_P==11 the new gesture data KH_II from Kristine Hein (FH Cologne).
% - or with pars.GENERATE_P==12 the iPhone gesture data from Daniel Bertram (FH Cologne).
%
% We do pars.nfold cross-validation.
%
% INPUT
%   opts        struct with several parameter settings for SFA_CLASSIFY. If omitted on
%               input, defaults will be used (see optsDefault() and source code)
%   pars        struct with several parameter settings for training and test data generation. 
%               If omitted on input, defaults will be set (see parsDefault() and source code)
% OUTPUT
%   res         struct with the results from the *last* classify-run (fields
%               y, ytst, predT, predX, see classify.m) plus the fields
%       errtrn  1 x 2 matrix: error rate with / w/o SFA on training set (mean over all CV-runs)
%       errtst  1 x 2 matrix: error rate with / w/o SFA on test set (mean over all CV-runs)
%       allerrtrn   nfold x 2 matrix (results from each CV run)
%       allerrtst   nfold x 2 matrix (results from each CV run)
%       realclass   nrecord x 1: the true class for each record
%       cvpredict   nrecord x 1: the CV-predicted class for each record
%       confmat     the confusion matrix of CV-predictions (columns=true class)
%                   confmat is the sum of all confmats for each CV-fold 
%       
% Remark: In the case pars.ncopies>0, res.errtrn contains the error rate on
%   all training patterns, including the parametric bootstrap records. One might
%   want to see the error of only the 'real' training records, but in most
%   cases with marginal data (where ncopies>0 is needed) the training error is 0 anyhow. 
%
% Example settings for struct pars:
%       {GENERATE_P=10; PERSON_ID=-1; nfold=10; tstvarseed=1; ncopies=0;}
%
% Author: Wolfgang Konen, Sep 2009 - Jan 2010
%
% See also: SFA_CLASSIFY, CLASS_DEMO2

    if nargin<2
        pars=parsDefault();
        pars.GENERATE_P=10;  % =2: load 'vowel-context.data' (UCI repository)  
                             % =3: load 'vowel-train.csv', 'vowel-test.csv' (Renal assignment)
                             % =4: load 'wdbc.data (UCI repository, malignant & benign breast cancer)
                             % =9: load 'class_pat_gest.mat' (gesture data Polak/Hein)
                             % =10: load 'hk_gesture_all-norm*.mat' (KH_I gesture data K.Hein) 
                             % =11: load 'hk_gesture_set*-norm*.mat' (KH_II gesture data K.Hein) 
                             % =12: load 'iphonegesture-norm*.mat' (iphone data D. Bertram, K.Hein) 
        pars.tstvarseed=0;   % =1: vary seed, =0: always seed 0, for random number generator
                             % in case of cross validation index 
    end
    if nargin<1
        opts=optsDefault();
        opts.gaussprj=0;     % 0/1: don't/do skip the PCA-preprocessing step for 'w/o SFA' (def. 0)
        opts.pp_range = 12;  % dims after preproc
        opts.dographics=0;   % =0: no plots, =1: some plots, =2: more plots
        opts.verbose=1;
    end
    if ~isfield(opts,'verbose')
       opts.verbose=2;     
    end

    [M,c_col,i_col,origin,info_preproc,opts] = dataLoadCV(opts,pars);

    [cvi,pars] = cvi_create(pars,size(M,1));   % cross validation over pars.nfold random subsets 
                                               % or over each person, if pars.CV_ON_PERSONS==TRUE

    % only needed for comparision with R and RF:
    % (see <fiwa_soma>\src\trunk\R-DM-Template\ClassifyTemplate\main_gesture.r & sweep_gesture.r)
    write_gestureCV_csv(cvi,M,pars);
    
    tic;
    %
    % run the pars.nfold (=10) folds through SFA
    % 
    allerrtrn = [];
    allerrtst = [];
    cases_trn = [];
    cases_tst = [];
    allrealclass = M(:,c_col);  % true class labels on all 'real' records (not noisy pars.ncopies records)
    allcvpredict =0*M(:,c_col);
    pclasserr = [];             % class errors for each fold
    cviclasses = unique(cvi);
    for i=1:length(cviclasses)
        vprintf(1,opts.verbose,['**** CV-fold ' num2str(i) ' ****\n']);
        CVICLASS = cviclasses(i);
        idxtst = find(cvi==CVICLASS);
        idxtrn = find(cvi~=CVICLASS);
        x    = M(idxtrn,i_col);             % training data
        xtst = M(idxtst,i_col);             % test data
        realclass = M(idxtrn,c_col)';    
        realc_tst = M(idxtst,c_col)';   

        % add pars.ncopies distorted copies, but only to the training data
        if pars.ncopies>0
            if pars.bootstrap==1
                % add noisy copies (Gaussian noise around class centroid)
                [realclass, x] = add_noisy_copies(realclass,x,pars);
            elseif pars.bootstrap >= 2
                % add random copies, randomly rotated and/or shortened
                if pars.ncopies > 0
                    [realclass,x] = add_rotated_copies(realclass,x,pars);
                end
            end
        end

        %
        % the main call to SFA classification
        %
        res = sfa_classify(x,realclass,xtst,realc_tst,opts,pars);
        
        %fprintf('training set size: %d\n',size(x,1));
        allerrtrn = [allerrtrn; res.errtrn];
        allerrtst = [allerrtst; res.errtst];
        cases_trn = [cases_trn; size(x,1)];
        cases_tst = [cases_tst; size(xtst,1)];
        allcvpredict(idxtst) = res.predT;
        [confmat,classerr] = mk_confmat(allrealclass(idxtst),res.predT,opts.nclass,unique(allrealclass));
        if opts.verbose>=2 
            confmat
            classerr
        end
        pclasserr = [pclasserr; classerr];
        i;
    end
    toc
    
    % calculate confusion matrix and error rate over *all* CV-folds:
    %
    [confmat,classerr] = mk_confmat(allrealclass,allcvpredict,opts.nclass);
    fprintf('CV confusion matrix & class errors in %%\n    --- true class ---\n')
    disp(confmat);
    sz=size(confmat);
    confmatRel = confmat ./ repmat(sum(confmat),sz(1),1)*100;   % relative confusion in percent
    disp(confmatRel);
    disp(['  ' num2str(classerr(1:opts.nclass)*100,'%6.1f') ...
          '%   Total: ' num2str(classerr(opts.nclass+1)*100,'%6.2f%%')]);
    
    res.allerrtrn = allerrtrn;
    res.allerrtst = allerrtst;
    w_trn = cases_trn/sum(cases_trn);
    w_tst = cases_tst/sum(cases_tst);
    if size(allerrtrn,1)>1
        res.errtrn = sum(repmat(w_trn,1,2) .* allerrtrn);
        res.errtst = sum(repmat(w_tst,1,2) .* allerrtst);
    end
    res.realclass = allrealclass;   % true class labels on all 'real' records 
    res.cvpredict = allcvpredict;   % CV predictions on these records
    res.confmat = confmat;          % confusion matrix (sum over all nfold CV folds)
    res.confmatRel = confmatRel;    % relative confusion matrix (percentage of each true class)
    res.info_preproc = info_preproc;
    res.pclasserr = pclasserr;
    res.opts = opts;
    res.pars = pars;
    res.pars.origin = origin;
    
  
    fprintf('                                with SFA   w/o SFA\n');    
    fprintf('CV error rate on training set = %8.3f%%  %8.3f%%\n', res.errtrn*100);    
    fprintf('CV error rate on     test set = %8.3f%%  %8.3f%%\n', res.errtst*100); 
	cverr=res.errtst(1);
	fprintf('CV_ERROR: %8.3f%%\n', cverr*100);
    %numel(find(allrealclass~=allcvpredict))/numel(allrealclass)

end 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% some private helper functions
%
function [cvi,pars] = cvi_create(pars,sz)
    if pars.CV_ON_PERSONS
        %
        % create cross validation index CVI over different persons 
        % (each person is test set in one run)
        %
        cvi = info_preproc(:,1)'-120;    
        pars.nfold = numel(unique(cvi));
    else 
        %
        % create random crossvalidation index CVI with pars.nfold folds
        %
        pars.nfold = 10.;
        cvi = zeros(1,sz);
        ilow=1;
        for i=1:pars.nfold
            ihigh = fix(sz/pars.nfold*i);
            cvi(ilow:ihigh) = i;        % i.e. [111..2222...333...]
            ilow = ihigh+1;
        end
        if pars.tstvarseed==0
            if ~isfield(pars,'seed'), pars.seed=4; end
            rand('state',pars.seed);        % each time the same random numbers
        else
            rand('state',sum(100*clock));   % each time different random numbers
        end    
        cvi = cvi(randperm(sz));        % permute CVI randomly 
    end
end

function write_gestureCV_csv(cvi,M,pars)
    % write the matrix of exactly this CV-division of
    % all data on file for comparision with R and RF (random
    % forest). First column: CV index.
    %path='/home/patrick/Desktop';
    path='datsave';
    fname=['/gestureCV_pat.' num2str(pars.n,'%02d') '.csv'];
    dlmwrite([path fname],[cvi' M],';');
    fprintf('Gesture data written onto %s\n',fname);
end 


