function res = class_demo2(opts,pars)
% class_demo2: demonstrate how classification can be done with SFA
%
% see [Berkes05] Pietro Berkes: Pattern recognition with Slow Feature
% Analysis. Cognitive Sciences EPrint Archive (CogPrint) 4104, http://cogprints.org/4104/ (2005)
%
% Instead of handwritten digits we use the Vowel or WDBC benchmark dataset 
% from UCI machine learning repository (http://archive.ics.uci.edu/ml) 
% or gesture data (Polak/Hein, FH Kln) from Cologne University
%
% INPUT
%   opts        struct with several parameter settings for SFA_CLASSIFY. If omitted on
%               input, defaults will be used (see optsDefault() and source code)
%   pars        struct with several parameter settings for training and test data generation. 
%               If omitted on input, defaults will be set (see parsDefault() and source code)
% OUTPUT
%   res         struct with the results from classify-run, e.g. 
%       errtrn  1 x 2 matrix: error rate with / w/o SFA on training set 
%       errtst  1 x 2 matrix: error rate with / w/o SFA on test set 
%       y       output from SFA when applied to training data
%       ytst    output from SFA when applied to test data
%       predT   predictions with SFA + GaussClassifier on test set
%       predX   predictions w/o SFA (only GaussClassifier) on test set 
%
% Example settings for struct pars:
%       {GENERATE_P=2; tstdiv=0; tstfrac=0.1; tstvarseed=0; ncopies=0;}
%       {GENERATE_P=9; tstdiv=1; tstfrac=0.1; tstvarseed=0; ncopies=1;}
%       {GENERATE_P=10; tstdiv=1; tstfrac=0.1; tstvarseed=0; ncopies=1;}
%
% Author: Wolfgang Konen, May 2009 - Nov 2011
%
% See also: CLASS_DEMO2A, CLASS_SWEEP, CLASS_CVDEMO2, dataLoad
%           SFA_CLASSIFY, sfaClassModel, sfaClassPredict

    if nargin<2
        pars = parsDefault();
        pars.GENERATE_P=10;  % =1: load 'Vowel.csv' (R mlbench), divide randomly in tst/trn
                             % =2: load 'vowel-context.data' (UCI repository, recommended)  
                             % =3: load 'vowel-train.csv', 'vowel-test.csv' (Renal assignment)
                             % =4: load 'wdbc.data (UCI repository, malignant & benign breast cancer)
                             % =9: load 'class_pat_gest.mat' (gesture data Polak/Hein)
                             % =10: load 'hk_gesture_all-norm*.mat' (KH_I gesture data K.Hein) 
                             % =11: load 'hk_gesture_set*-norm*.mat' (KH_II gesture data K.Hein) 
                             % =12: load 'iphonegesture-norm*.mat' (iphone data D. Bertram, K.Hein) 
        pars.PERSON_ID=  -1; % Only for pars.GENERATE_P==10,11,12:
                             % -1: all persons, +N: only records of a specific person, 
                             % -N: all records of persons <= |N|, 
                             % with N=121,...,130 for pars.GENERATE_P=10,11
                             % and N=1,2,4,5,..,15 for pars.GENERATE_P=12
    end
    if nargin<1
        opts = optsDefault();
    end

    [x,realclass,xtst,realc_tst,opts] = dataLoad(opts,pars);

    res = sfa_classify(x,realclass,xtst,realc_tst,opts,pars);

    % confusion matrix & class error on test set
    [confmat,classerr] = mk_confmat(realc_tst,res.predT,opts.nclass,opts.classes);
    fprintf('Confusion matrix & class errors on test set\n    --- true class ---\n')
    disp(confmat);
    disp(num2str(classerr,'%7.3f'));
    
    res.opts = opts;
    res.pars = pars;
end 

