function res = class_demo2A(opts,pars)
% class_demo2A: demonstrate how a SFA classification model is build and stored
%               (see the fields opts.*Filename below)
%
% class_demo2A + class_demo_2B do the same as class_demo2, 
% class_demo2A does only the training part, class_demo2B only the
% prediction part. The difference to class_demo2 is that all trained model
% elements are stored on file(s) and thus training and prediction can be invoked
% independently.
%
% Instead of handwritten digits we use the Vowel or WDBC benchmark dataset 
% from UCI machine learning repository (http://archive.ics.uci.edu/ml) 
% or gesture data (Polak/Hein, FH Kln) from Cologne University, see
% dataLoad.m
%
% INPUT
%   opts        struct with several parameter settings for SFA_CLASSIFY. If omitted on
%               input, defaults will be used (see optsDefault() and source code)
%   pars        struct with several parameter settings for training and test data generation. 
%               If omitted on input, defaults will be set (see parsDefault() and source code)
% OUTPUT
%   res         struct with the results from classify-run, e.g. 
%       errtrn  1 x 2 matrix: error rate with / w/o SFA on training set 
%       errtst  1 x 2 matrix: error rate with / w/o SFA on test set 
%       y       output from SFA when applied to training data
%       ytst    output from SFA when applied to test data
%       predT   predictions with SFA + GaussClassifier on test set
%       predX   predictions w/o SFA (only GaussClassifier) on test set 
%
% SIDE EFFECTS
%   Saves SFA model on datsave/ClassifierSFA.mat
%   Saves GAUSS classifier for SFA in datsave/ClassifierGauss.mat
%   Saves GAUSS classifier X in datsave/ClassifierXGauss.mat
%   Saves {x, xtst, realclass, realc_tst, opts} in datsave/Classif_class_demo2A.mat
%
% Example settings for struct pars:
%       {GENERATE_P=2; tstdiv=0; tstfrac=0.1; tstvarseed=0; ncopies=0;}
%       {GENERATE_P=9; tstdiv=1; tstfrac=0.1; tstvarseed=0; ncopies=1;}
%       {GENERATE_P=10; tstdiv=1; tstfrac=0.1; tstvarseed=0; ncopies=1;}
%
% Author: Wolfgang Konen, Jan-Aug 2011
%
% See also: sfaClassModel, CLASS_DEMO2, CLASS_DEMO2B, dataLoad 

    if ~isdir('datsave'), mkdir('datsave');, end;
    if nargin<2
        pars=parsDefault();
        pars.GENERATE_P=11;  % =1: load 'Vowel.csv' (R mlbench), divide randomly in tst/trn
                             % =2: load 'vowel-context.data' (UCI repository, recommended)  
                             % =3: load 'vowel-train.csv', 'vowel-test.csv' (Renal assignment)
                             % =4: load 'wdbc.data (UCI repository, malignant & benign breast cancer)
                             % =9: load 'class_pat_gest.mat' (gesture data Polak/Hein)
                             % =10: load 'hk_gesture_all-norm*.mat' (KH_I gesture data K.Hein) 
                             % =11: load 'hk_gesture_set*-norm*.mat' (KH_II gesture data K.Hein) 
                             % =12: load 'iphonegesture-norm*.mat' (iphone data D. Bertram, K.Hein) 
        pars.PERSON_ID=  -1; % Only for pars.GENERATE_P==10,11,12:
                             % -1: all persons, +N: only records of a specific person, 
                             % -N: all records of persons <= |N|, 
                             % with N=121,...,130 for pars.GENERATE_P=10,11
                             % and N=1,2,4,5,..,15 for pars.GENERATE_P=12
    end
    if nargin<1
        opts=optsDefault();
        opts.dographics=3;   % =0: no plots, =1: some plots, >=2: more plots
    end

    [x,realclass,xtst,realc_tst,opts] = dataLoad(opts,pars);
    
    res = sfaClassModel(x,realclass,opts);
    % writes results onto .mat files opts.sFilename, opts.gFilename,
    % opts.xFilename

    % confusion matrix & class error on training set
    [confmat,classerr] = mk_confmat(realclass,res.predC,opts.nclass);
    fprintf('Confusion matrix & class errors on training set\n    --- true class ---\n')
    disp(confmat);
    disp(num2str(classerr,'%7.3f'));
    
    res.opts = opts;
    res.pars = pars;
    
    fname='datsave/Classif_class_demo2A.mat';
    save(fname,'x','xtst','realclass','realc_tst','opts');
    fprintf('{x, xtst, realclass, realc_tst, opts} saved in %s\n',fname);

end 

