function [predC,prob] = gaussClassifier(hdl,y,realC,method,opts_CL)
% gaussClassifier: train or apply a Gaussian or Nearest Neighbour 
% classifier HDL, which has been created before with gaussCreate.
% INPUT:
%   y       K x M matrix where K is the total number of patterns and M is the
%           number of variables used for classification. I.e. each row of
%           y contains the data for one pattern.
%   realC   1 x K matrix with NCLASS distinct real class labels needed only
%           for method='train'. In case of method='apply' realC is not 
%           used and can have any value
%   method  either 'train' (def.) or 'apply'
%   opts_CL the struct opts.CL, containing the fields:
%     aligned =0: do not align the Gaussian classifiers with axes, use full
%           covariance matrix
%           =1 (def.): set the off-diagonals in covariance matrix to 0,
%           i.e. the Gaussian classifier is forced to be aligned with the
%           axes. This is more robust in the case where the data deviate
%           largely from a multivariate normal distribution. 
%     algo  either 'gauss' (def.) or 'nearneig' (nearest neigbour)
%     epsD  [0.04] replace diagonal elements of COV smaller than epsD with
%           epsD to avoid too small Gaussians (only for algo='gauss')
% OUTPUT: 
%   predC   1 x K matrix: the predicted class = argmax(prob(k,:))
%   prob    K x NCLASS matrix: prob(k,n) is the estimated probability that
%           pattern k belongs to class m
%
% Author: Wolfgang Konen, Cologne Univ. of Applied Sciences, Aug 2009 - Nov 2011
%
% See also: GAUSSCREATE, GAUSS_CLEARALL

    global GAUSS_STRUCTS    
    nclass = GAUSS_STRUCTS{hdl}.nclass; 
    if ~exist('method','var'), method='train';, end;
    if isfield(opts_CL,'algo'), algo=opts_CL.algo;, else algo='gauss';, end;
    if isfield(opts_CL,'aligned'), aligned=opts_CL.aligned;, else aligned=1;, end;
    if isfield(opts_CL,'epsD'), epsD=opts_CL.epsD;, else epsD=0.01;, end;

    if size(y,2)~=size(GAUSS_STRUCTS{hdl}.X0,2) 
        disp('Size mismatch y <--> GAUSS_STRUCTS{hdl}.X');
    end
    
    if strcmp(method,'train')
        GAUSS_STRUCTS{hdl}.uni = unique(realC);        % the unique labels 
        % (during training we must have at least one record from each
        % class, during 'apply' there can be fewer class labels present, 
        % therefore we store GAUSS_STRUCTS{hdl}.uni for later reference)
        if strcmp(algo,'gauss')
            trainGauss(hdl,y,realC,aligned);
        elseif strcmp(algo,'nearneig')
            trainNearNeighbour(hdl,y,realC);
        else
            error(['algo = ' algo ' is not allowed']);
        end
        GAUSS_STRUCTS{hdl}.isTrained=1;
    elseif strcmp(method,'apply')
    else
        error(['method = ' method ' is not allowed']);
    end % strcmp(method,...)
    
    if GAUSS_STRUCTS{hdl}.isTrained==0
        error(sprintf('Gaussian Classifier hdl=%d is not trained >> cannot apply it',hdl));
    end
    % 
    % this is done for both methods, 'train' and 'apply':
    %
    if strcmp(algo,'gauss')
        [predC,prob] = applyGauss(hdl,y);
    elseif strcmp(algo,'nearneig')
        [predC,prob] = applyNearNeighbour(hdl,y);
    end
    
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   function trainGauss(hdl,y,realC,aligned)
        first = 1; 
        for n=1:nclass
            ind = find(realC==GAUSS_STRUCTS{hdl}.uni(n));
            GAUSS_STRUCTS{hdl}.X0(n,:) = mean(y(ind,:));
            arg = y(ind,:)-repmat(GAUSS_STRUCTS{hdl}.X0(n,:),numel(ind),1);
            COV = arg' * arg/(numel(ind)-1);
            if (epsD>0)
                dcov = diag(COV);
                if (min(dcov)<epsD & first==1)
                    first=0;
                    warning(['Warning: Some diagonal elements of COV are smaller than ' num2str(epsD) ...
                             ' --> replacing them with ' num2str(epsD)]);
                end
                dcov(dcov<epsD) = epsD;
                COV = COV - diag(diag(COV)) + diag(dcov);
            end
            if aligned
                % zero all off-diagonal elements >> ensures that the densitiy
                % function has ellipsoidal equipotential surfaces and that
                % the ellipsoid axes are *aligned* with the coordinate axes
                COV = diag(diag(COV));
            end
            GAUSS_STRUCTS{hdl}.COV(:,:,n) = COV;
            GAUSS_STRUCTS{hdl}.iCOV(:,:,n) = inv(GAUSS_STRUCTS{hdl}.COV(:,:,n));
            GAUSS_STRUCTS{hdl}.f0(n) = 1/(2*pi)^(GAUSS_STRUCTS{hdl}.odim/2) ...
                                        /sqrt(det(GAUSS_STRUCTS{hdl}.COV(:,:,n)));
            GAUSS_STRUCTS{hdl}.P_c(n) = numel(ind)/numel(realC);
            
            DBG=0;
            if DBG>0 
                [X1,X2] = meshgrid(-2:.2:2,-2:.2:2);
                Z = 0*X1;
                if GAUSS_STRUCTS{hdl}.odim>2
                    X3 = GAUSS_STRUCTS{hdl}.X0(n,3:end);
                else 
                    X3 = [];
                end
                for i=1:size(X1,1)
                    for j=1:size(X1,2)
                        xvec=[X1(i,j) X2(i,j) X3] - GAUSS_STRUCTS{hdl}.X0(n,:); 
                        Z(i,j) = exp(-xvec*GAUSS_STRUCTS{hdl}.iCOV(:,:,n)*xvec'/2);                      
                    end
                end
                Z = Z*GAUSS_STRUCTS{hdl}.f0(n);
                fprintf('sum(Z)=%g\n',sum(Z(:)));
                figure(2); clf; 
                hold on;
                contour(X1,X2,Z);
                plot(y(ind,1),y(ind,2),'.r');
                hold off;
                xlabel('y_1'); ylabel('y_2');
                xlim([-2 2]); ylim([-2 2]);
                set(gca, 'PlotBoxAspectRatio', [1,1,1]);
            end % DBG>0
        end
    end %function trainGauss
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    function [predC,prob] = applyGauss(hdl,y)
        prob = zeros(size(y,1),nclass);
        for k=1:size(y,1)
            sum_Py=0;
            for n=1:nclass
                arg = y(k,:)-GAUSS_STRUCTS{hdl}.X0(n,:);
                Py_c(n) = GAUSS_STRUCTS{hdl}.f0(n) * exp(-arg*GAUSS_STRUCTS{hdl}.iCOV(:,:,n)*arg'/2);
                sum_Py = sum_Py+Py_c(n)*GAUSS_STRUCTS{hdl}.P_c(n);
            end
            if sum_Py==0
                % the pattern y(k,:) is so far away from all Gaussians that we
                % can't tell the class >> random guess
                Pc_y = rand(1,nclass);
                warning(['sum_Py==0, pattern far outside for k=' num2str(k)]);
            else
                Pc_y = Py_c .* GAUSS_STRUCTS{hdl}.P_c' / sum_Py;
            end
            predC(k) = GAUSS_STRUCTS{hdl}.uni(find(Pc_y==max(Pc_y(:))));
            prob(k,:) = Pc_y;
        end
    end %function applyGauss

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    function trainNearNeighbour(hdl,y,realC)
        GAUSS_STRUCTS{hdl}.YTR = y;
        GAUSS_STRUCTS{hdl}.cTR = realC;
    end %function trainNearNeigbour
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    function [predC,prob] = applyNearNeighbour(hdl,y)
        nTR = size(GAUSS_STRUCTS{hdl}.YTR,1);
        prob = zeros(size(y,1),nclass);
        for k=1:size(y,1)
            sum_Py=0;
            arg = repmat(y(k,:),nTR,1)-GAUSS_STRUCTS{hdl}.YTR;
            arg = sum(arg' .* arg');     % now arg(i) is the squared distance to the i-th training record
            for n=1:nclass
                ind = find(GAUSS_STRUCTS{hdl}.cTR==GAUSS_STRUCTS{hdl}.uni(n));
                Py_c(n) = 1/(min(arg(ind))+1e-10);
                sum_Py = sum_Py+Py_c(n); %*GAUSS_STRUCTS{hdl}.P_c(n);
            end
            if sum_Py==0
                error(['sum_Py==0 for k=' num2str(k)]);
            else
                Pc_y = Py_c / sum_Py;
            end
            predC(k) = GAUSS_STRUCTS{hdl}.uni(find(Pc_y==max(Pc_y(:))));
            prob(k,:) = Pc_y;
        end
    end %function applyNearNeigbour
    
end