function [W,DW,D,kvar]=lcov_pca2(handle, dim_range)
% LCOV_PCA2 Principal Component Analysis on a covariance object.
%   [W,DW,D,kvar] = LCOV_PCA2(HANDLE[, DIM_RANGE]) performs PCA _and_ whitening
%   on the covariance object referenced by HANDLE.
%   W is the whitening matrix, DW the dewhitening matrix and D an array
%   containing a list of the eigenvalues. kvar contains the total
%   variance kept in percent.
%
%   ! LCOV_FIX must have been called _before_ you call this function.
%  
%   If DIM_RANGE is specified, perform dimensionality reduction:
%       if it is a number: only the first components 1:DIM_RANGE are kept
%           (those with largest eigenvalues)
%       if it is a range: only the components in the range DIM_RANGE(1)..DIM_RANGE(2) are kept
%
% - Difference to LCOV_PCA: null the rows of W (columns of DW) where the 
% - corresponding eigenvalue in D is close to zero (more precisely: if
% - lam/lam_max < EPS = 1e-7). This is numerically stable in the case where
% - the covariance matrix is singular.
% -
% - Author: Wolfgang Konen, Cologne Univ. , May'2009
%
%   See also LCOV_FIX, LCOV_PCA.
  
  global LCOV_STRUCTS
  EPS = 1e-7;
  
  if nargin<2, dim_range=length(LCOV_STRUCTS{handle}.COV_MTX); end
  if length(dim_range)==1, dim_int=1:dim_range;
  else dim_int=dim_range(1):dim_range(2); end

  [tmp,D,PC]=svd(LCOV_STRUCTS{handle}.COV_MTX);
  % svd sorts the eigenvalues by itself (in *decreasing* order)
  
  %reduce the dimensionality (if numel(dim_int) < size of covariance matrix)
  D=diag(D);
  lammax=D(1); 
  kvar = sum(D(dim_int))/sum(D);
  D=D(dim_int);
  Dmtx=D.^(-0.5);
  Dmtx(D<lammax*EPS)=0.0;       % This is the important point: if an eigenvalue d is 
                                % zero or very small, then set 1/d to zero as well,
                                % i.e. set the corresponding row in matrix
                                % W to zero, because it would only contribute
                                % amplified numeric errors.
  Dmtx=diag(Dmtx);
    
  PC=PC(:,dim_int)';
   
  W=Dmtx*PC;
  DW=PC'*diag(D);
