function [realclass, x] = add_noisy_copies(realclass,x,pars)
% add_noisy_copies: Given training data X with true labels REALCLASS, 
% add new records to X and REALCLASS, which are noisy copies of the training data. 
% Parameters:
%   pars.ncopies    The number of new records to add
%   pars.ncsigma    The noise in each column of x has the std.dev.
%                   pars.ncsigma*(standard deviation of column)
%   pars.ncmethod   =1: each 'old' record from X in turn is the centroid
%                       for a new pattern;
%                   =2: the centroid is the average of all records from the
%                       same class, the std.dev. is the same for all classes;
%                   =3: centroid as in '2', the std.dev. is the std.dev. of
%                       all records from the same class  (*recommended*)
%
% Author: Wolfgang Konen, Aug 2009 - Nov 2011
%
% See also: sfaPBootstrap, CLASS_DEMO2, CLASS_CVDEMO2, CLASS_SWEEP
    ncopies = pars.ncopies;
    if ~isfield(pars,'ncsigma'), pars.ncsigma=1.0; end;
    if ~isfield(pars,'ncmethod'), pars.ncmethod=3; end;

    if ncopies>0
        % add noisy copies
        x0 = x;
        r0 = realclass;
        c0 = get_centroid(x0,r0,pars.ncmethod);
        s0 = get_sigma(x0,r0,pars.ncmethod);
        
        sz1 = size(x0,1);           % number of training records in x0
        sz2 = size(x0,2);
        if sz1==0, error('No training data in x!');, end;
        nn1 = fix(ncopies/sz1)+1;   % example: ncopies=220, sz1=100 --> nn1=3
        % We divide the generation of the ncopies bootstrap patterns
        % into nn1 loop steps. This division is only needed pars.ncmethod=1
        % (each training record in turn serves as centroid). If
        % pars.ncmethod=2, we could as well generate the bootstrap patterns
        % in one step.
        for k=1:nn1
            x2 = c0 + pars.ncsigma * s0 .* randn(sz1,sz2);
            % i.e., calculate for each column of x0 the standard deviation,
            % repeat it sz1 times, multiply each entry by a random number
            % with standard normal distribution, multiply by pars.ncsigma
            % --> as a result we have
            %       std(x2-c0) = pars.ncsigma * std(x0)
            % (approximately, up to statistical fluctuations).
            % The tacit assumption is here that the standard deviations in
            % each column are the same for all classes. 
            
            if k==nn1
                % in the last pass, add only that many randomly selected
                % records from x2 that the total number of added records 
                % amounts exactly to pars.ncopies
                idx = randperm(sz1);
                idx = idx(1:mod(ncopies,sz1));
                x2 = x2(idx,:);
                r0 = r0(idx);
            end
            x = [x; x2];
            realclass = [realclass r0];
        end
        % sort the training data class-by-class
        [tmp,idx] = sort(realclass);
        realclass = realclass(idx);
        x = x(idx,:);
    end
end

function c0 = get_centroid(x0,r0,ncmethod)
% ncmethod==1: c0 = x0 (each pattern is its own centroid)
% ncmethod==2: c0 = centroid(m(x0)) where m(x0) is the class to which x0 belongs
    if (ncmethod==1) 
        c0=x0;          % faster, but not exactly 'parametric bootstrap'
    else
                        % we recommend this method since it adds new
                        % patterns which are statistically independent from
                        % x0 (true parametric bootstrap)
        classes=unique(r0);
        c0 = x0*0;
        for i=1:numel(classes)
            ind = find(r0==classes(i));
            c0(ind,:) = repmat(mean(x0(ind,:)), numel(ind), 1);
        end
    end
end

function s0 = get_sigma(x0,r0,ncmethod)
% ncmethod <3: s0 = std(x0) (each column has a common sigma, std over all classes)
% ncmethod==3: s0 = std(m(x0)) where m(x0) is the class to which x0 belongs
    sz1 = size(x0,1);           % number of training records in x0
    if (ncmethod<3) 
        s0=repmat(std(x0),sz1,1);          % faster, but perhaps not precise modeling 
    else
        classes=unique(r0);
        s0 = x0*0;
        for i=1:numel(classes)
            ind = find(r0==classes(i));
            s0(ind,:) = repmat(std(x0(ind,:)), numel(ind), 1);
        end
    end
end
