% nl_regress.m: Perform non-linear regression.
%
% Given the data in ARG, expand them nonlinearly in the same way as it was
% done in the SFA-object HDL (expanded dimension M) and search the vector
% RCOEF of M constant coefficients, such that the sum of squared residuals 
% between a given function in time FUNC and the function
%       R(t) = (v(t) - v0)' * RCOEF,        t=1,...,T,
% is minimal  
%  
% INPUT:
%       hdl             handle to a SFA_STRUCTS object
%       arg             (T x m) the embedded data s(t) row-by-row, they are 
%                       transformed into v(t) the same way as in SFA
%       func            (T x 1) the function to be fitted nonlinearly 
% OUTPUT: 
%       R               (T x 1) the function fitted by NL-regression
%       rcoef           (M x 1) the coefficients for the NL-expanded
%                       dimensions
%
% Author: Wolfgang Konen, Dec 2009
%
function [R, rcoef] = nl_regress(hdl,arg,func)
    global SFA_STRUCTS
  
    % project data and reduce dimensionality
    arg=arg-repmat(SFA_STRUCTS{hdl}.avg0, size(arg,1), 1);
    % expand data
    arg=expansion(hdl, arg*SFA_STRUCTS{hdl}.W0');
    arg=arg-repmat(SFA_STRUCTS{hdl}.avg1,size(arg,1),1);
    % now ARG contains (v(t)-v0)' (rows: t, columns: expanded space dim M)

    fmat = arg .* repmat(func,1,size(arg,2));
    d_f = (mean(fmat))';        % column mean
    rcoef = pinv(SFA_STRUCTS{hdl}.myB)*d_f;
    % the above line is for regular B (=myB) equivalent to 
    %    rcoef = SFA_STRUCTS{hdl}.myB \ d_f;
    % But pinv() works better, if B is singular
    R = arg * rcoef;
end