function sfa2_dbg(hdl,arg,DG,S,DS,BD,W1,D1) 
% SFA2_DBG is currently called only when method='SVD_SFA' and dbg>0 in
% SFA2_STEP.M
%
% SFA2_DBG makes several debug checks during the SFA-step of the SFA2
% algorithm. It compares the SFA-TK method from [Berkes05] ('GEN_EIG',
% generalized eigenvectors) with the alternative approach from [WisSej02]
% ('SVD_SFA', sphere expanded data, then timediff).
%
% INPUT
%   hdl     the handle to the SFA object
%   arg     the preprocessed, expanded, mean-free data (not yet sphered)
%   DG      the diagonal matrix of eigenvalues from 'GEN_EIG'-approach
%   S,DS    the sphering / de-spering matrix for expanded data in 'SVD_SFA' 
%   BD      eigenvalues of expanded covariance matrix B (from lcov_pca2)
%   W1      columns of W1 contain eigenvectors of Cov(\dot z) from 'SVD_SFA'
%   D1      diagonal of D1 has the EW of Cov(\dot z) from 'SVD_SFA'
%
% OUTPUT    only printed output on console
%
% Author: Wolfgang Konen, Cologne Univ. , 05/2009
%
    global SFA_STRUCTS
    global LCOV_STRUCTS
    
    xpsize=SFA_STRUCTS{hdl}.xp_range;
    xphdl=SFA_STRUCTS{hdl}.xp_hdl;
    dfhdl=SFA_STRUCTS{hdl}.diff_hdl;
    Cp=LCOV_STRUCTS{dfhdl}.COV_MTX;     % Cp = C' = Cov(\dot v)
    C = S*Cp*S';                        % C = Cov(\dot z)
    B=LCOV_STRUCTS{xphdl}.COV_MTX;
    W=SFA_STRUCTS{hdl}.SF;      % columns of W contain eigenvectors w_j from 'GEN_EIG'
    
    % the pair [W,DG] (from 'GEN_EIG) is not sorted by increasing eigenvalues.
    % Therefore we sort it here.
    DGG = diag(DG);         % from diagonal matrix to vector
    [tmp,idx]=sort(DGG);
    DGG = DGG(idx);
    DG = diag(DGG);         % from vector back to diagonal matrix
    W = W(:,idx);

    % the pair [W1,D1] (from 'SVD_SFA', svd) is not sorted by increasing eigenvalues.
    % Therefore we sort it here.
    D1 = diag(D1);
    [tmp,idx]=sort(D1);
    D1 = D1(idx);
    W1 = W1(:,idx);
    
    SF1 = S'*W1;
    
      OLD_VERSION=0;
      if OLD_VERSION==1
          % this is the old version of 'SVD_SFA' and is **not**
          % recommended since: (a) it is plain wrong for switch 'CLASSIF'
          % and (b) it does not allow chunkwise incorporation of data 
          % and (c) it is more costly to calculate.
          % It resides here only for debugging comparisions.
          warning('Using OLD_VERSION in SVD_SFA');
          xpsize=SFA_STRUCTS{hdl}.xp_range;
          dif2_hdl=lcov_create(xpsize);
          arg=arg-repmat(SFA_STRUCTS{hdl}.avg0, size(arg,1), 1);
          arg=expansion(hdl, arg*SFA_STRUCTS{hdl}.W0');
          arg=arg-repmat(SFA_STRUCTS{hdl}.avg1,size(arg,1),1);
          lcov_update(dif2_hdl, timediff(arg*S', SFA_STRUCTS{hdl}.ax_type));
          lcov_fix(dif2_hdl);
          C = LCOV_STRUCTS{dif2_hdl}.COV_MTX;
          lcov_clear(dif2_hdl);
          [W1,D1] = eig(C);
          SF1 = S'*W1;
      end

    % CHECK 1a: Do the columns of W ('GEN_EIG') solve the generalized eigenvalue
    % equation? If so, errmax should be as small as machine accuracy.
    err = Cp*W - B*W*DG; 
    % CAUTION: the order B*W*DG (with DG at end!!) is important. Only in
    % this way the jth column of err represents the residual vector
    %       Cp*W(:,j) - lam_j * B*W(:,j)
    % where lam_j = DG(j,j) is the jth eigenvalue.
    errmax = max(abs(err(:)));       % check: errmax should be as small as machine accuracy
    % CHECK 1b: Do the columns of W1 ('SVD_SFA') really solve the eigenvalue 
    % equation with C? If so, errmax2 should be as small as machine accuracy.
    err2 = C*W1 - W1*diag(D1); %DG;
    errmax2 = max(abs(err2(:)));     % check: errmax2 should be as small as machine accuracy

    % CHECK 2a: Are the eigenvalues of B, as calculated with lcov_pca2 or
    % with eig, the same? - If so, ev_max should be as small as machine
    % accuracy.
    % CHECK 2b: Is B ill-conditioned, i.e. has B a large condition number cn
    % (ratio of largest/smallest eigenvalue)? - If cn>1e9, the method 'GEN_EIG'
    % is likely to fail, while the method 'SVD_SFA' has shown to work up to
    % cn=1e15.
    [BV,BD2] = eig(B);
    ev = [sort(diag(BD2)) sort(BD)]; % check: both columns should be the same
    ev_max = max(abs(ev(:,1)-ev(:,2)));
    cn = abs(ev(numel(BD))/ev(1));   % Condition number of B
    disp(['sfa2_dbg: Condition number of B=Cov(v):  ' num2str(cn,'%12.3e') '   (should be below 1e9)']);
    
    % CHECK 3a: Is the covariance matrix of  the sphered expanded data (z) 
    % the unit matrix? - If so, then gammax should be as small as machine accuracy.
    z = arg*S';                 % the columns have norm^2=(size(arg,1)-1)
    z = z(:,sum(z)~=0);         % exclude those columns which are exactly 0 in S'
    za = z'*z./(size(arg,1)-1);
    za = za-eye(size(z,2));
    gammax=max(abs(za(:)));    
    % CHECK 3b: Are the slow signals y_j, which correspond to DIFFERENT
    % eigenvalues, orthonormal, if we take the expectation value
    % of their products as scalar product?  
    % Acc. to [WisSej02] they should be orthonormal. - If they are orthonormal,
    % then delmax should be as small as machine accuracy.
    idx = find(D1-circshift(D1,1) > 1e-9);      % sort out columns with same eigenvalue as predecessor
    y = arg*SF1(:,idx);
    y = y(:,sum(y)~=0);         % exclude those columns which are exactly 0 in SF1
    zb = y'*y./(size(arg,1)-1);
    zb = zb-eye(size(y,2));
    delmax=max(abs(zb(:)));    
    % CHECK 3c: Are the columns of W1 (eigenvectors of C), which correspond to DIFFERENT
    % eigenvalues, orthonormal? If so, delmax2 should be as small as machine accuracy.
    idx = find(D1-circshift(D1,1) > 1e-9);      % sort out columns with same eigenvalue as predecessor
    zz=W1(:,idx)'*W1(:,idx)-eye(numel(idx));
    delmax2=max(abs(zz(:)));
    rank_defect=size(W1,1)-rank(W1);       % should be zero (W1 should span the whole space)
    disp(['sfa2_dbg: [errmax ev_max gammax delmax] =   [' num2str([errmax ev_max gammax delmax],'%12.3e') ']   (should be all small)']);
    disp(['    errmax: do the columns of W (from GEN_EIG) solve the generalized EV-problem?']);
    disp(['    ev_max: are the EV of B=Cov(v) as calculated with lcov_pca2 and with eig the same?']);
    disp(['    gammax: is Cov(z) the unit matrix (z=sphered expanded training data, from SVD_SFA)?']);
    disp(['    delmax: are the slow signals (from SVD_SFA) orthonormal (in their expectation value)?']);
    disp(['sfa2_dbg: [errmax2 delmax2 rank_defect] =   [' num2str([errmax2 delmax2 rank_defect],'%12.3e') ']   (should be all small)']);
    disp(['    errmax2: do the columns of W1 (from SVD_SFA) solve the EV-problem for C?']);
    disp(['    delmax2: are the columns of W1 (eigenvectors of C, from SVD_SFA) orthonormal?']);
    disp(['    rank_defect: is =0, if rank(W1) = dim(W1)']);
    
    % CHECK 4: Comparing 'SVD_SFA' with 'GEN_EIG': Do the first ten slowest 
    % signals (seen as vectors in expanded space) point into the same direction 
    % (up to a minus sign)? - If so, the coefficient ratios should span a
    % min-max-range close to 1.0 (or -1.0).
    disp('min/max-range for the ratio of first ten eigenvectors (WisSej/Berkes):');
    rng = 1:min([10 xpsize]);
    del4 = min([max(abs(SF1(:,rng)+W(:,rng))); max(abs(SF1(:,rng)-W(:,rng)))]);
    disp(num2str(del4,'%10.1e'));
    %disp(num2str([min(SF1(:,rng)./W(:,rng));max(SF1(:,rng)./W(:,rng))],'%10.1e'));

    % CHECK 5: Does sphering of expanded data work? I.e. do the 
    % sphered data ARG*S' produce a covariance matrix CZ with all '1.0' on the 
    % diagonal? - If the data ARG are degenerate, then some eigenvalues of
    % CZ will be zero or very close to zero resulting in an infinite or at least 
    % very large condition number.
    xpsize=SFA_STRUCTS{hdl}.xp_range; %xp_dim(insize);
    zDATA = arg*S';             % sphere the expanded data
    zDATA = zDATA(:,sum(zDATA)~=0);  % exclude those columns which are exactly 0 in S'
    zp_hdl=lcov_create(size(zDATA,2));
    lcov_update(zp_hdl, zDATA);
    lcov_fix(zp_hdl);
    CZ = LCOV_STRUCTS{zp_hdl}.COV_MTX;
    za = CZ - eye(size(zDATA,2));
    cmax = max(abs(za(:)));
    dd = sort(diag(CZ));
    if dd(1)==0
        dd_cond=Inf;
    else
        dd_cond = abs(dd(numel(dd))/dd(1));
    end
    disp(['sfa2_dbg: Condition number of sphered expanded data z:   ' num2str(dd_cond,'%12.3e') '  (should be close to 1)']);
    disp(['sfa2_dbg: Deviation of Cov(z) from unit matrix:   ' num2str(cmax,'%12.3e') '  (should be close to 0)']);
    lcov_clear(zp_hdl);
end
