function res = sfaClassModel(x,realclass,opts)
% sfaClassModel: build & store SFA classification model:
%               1st part of sfa_classify, store results in opts.*Filename.
%               2nd part in sfaClassPredict (test set prediction)
%
% see [Berkes05] Pietro Berkes: Pattern recognition with Slow Feature Analysis.
% Cognitive Sciences EPrint Archive (CogPrint) 4104, http://cogprints.org/4104/ (2005)
%
% INPUT
%   x           NREC x IDIM, training input data
%   realclass   1 x NREC, training class labels
%   opts        struct with several parameter settings:
%       deg       degree of SFA model
%       gaussprj
%       ...
%       *Filename [* = s,g,x] where to store the models (see SIDE EFFECTS)
%
% OUTPUT
%   res         struct with fields
%       errtrn  1 x 2 matrix: error rate with / w/o SFA on training set 
%       y       output from SFA when applied to training data
%       predC   predictions with SFA + GaussClassifier on training data
%       predX   predictions w/o SFA (only GaussClassifier) on training data 
%
% SIDE EFFECTS
%       Trained SFA model is written on opts.sFilename
%       Trained GAUSS model with SFA is written on opts.gFilename
%       Trained GAUSS model w/o SFA is written on opts.xFilename
%
% Author: Wolfgang Konen, Jan 2011
%
% See also: sfaClassPredict, SFA_CLASSIFY, CLASS_DEMO2A, CLASS_DEMO2B 

if ~isfield(opts,'gaussprj')
   opts.gaussprj=0;     % 0/1: don't/do skip the PCA-preprocessing step for 'w/o SFA'
end
if ~isfield(opts,'deg')
   opts.deg=2;          % 1/2: create SFA1 or SFA2 object
end

idim    =opts.idim;
nclass  =opts.nclass;

%sfa_clearall;

trnnum=numel(realclass);        % number of training cases

if opts.dographics>0
    cmap = colormap(jet);
    colors=cmap(mod(15*(1:nclass),size(cmap,1))+1,:); 
    figure(1); clf; set(gcf, 'Position', [89 77 866 638]);
    spl = 0;    % subplot counter
end
if opts.dographics>=2
    % scatter plot input signal (first 2 dims, all classes in different
    % colors)    
    spl=spl+1; subplot(2,3,spl);   
    hold on; 
    for i=1:nclass
        ind = find(realclass==opts.classes(i));
        plot(x(ind,1),x(ind,2),'Marker','.','MarkerSize',5,'Color',colors(i,:),'LineStyle','none');
        xlabel('x_1'); ylabel('x_2');
    end 
    set(gca, 'PlotBoxAspectRatio', [1,1,1]);
    hold off;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Slow Feature Analysis
%
% create a SFA object
if opts.deg==2
    hdl = sfa2_create(opts.pp_range, xp_dim(opts.pp_range), 'PCA2', 'ORD1', 0, opts);
    % perform the preprocessing step
    sfa_step(hdl, x, 'preprocessing');
    % perform the expansion step 
    % IMPORTANT: add patterns one class at a time (!)
    for n=1:nclass
        sfa_step(hdl, x(realclass==opts.classes(n),:), 'expansion','CLASSIF');
    end
    % close the algorithm
    sfa_step(hdl, x, 'sfa','SVD_SFA');        
    %sfa_step(hdl, x, 'sfa','GEN_EIG');      
else % i.e. opts.deg==1
    hdl = sfa1_create(opts.pp_range, 'ORD1', 0);       %--- does not work yet
    % perform the preprocessing step 
    % IMPORTANT: add patterns one class at a time (!)
    for n=1:nclass
        sfa_step(hdl, x(realclass==opts.classes(n),:), 'preprocessing','CLASSIF');
    end
    % close the algorithm
    sfa_step(hdl, x, 'sfa','SVD_SFA');        
    %sfa_step(hdl, x, 'sfa','GEN_EIG');    
end
sfa_save(hdl,opts.sFilename);
fprintf('SFA model saved in %s\n',opts.sFilename);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% compute the SFA output signal on the training data
y = sfa_execute(hdl, x);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% train the Gaussian classifier using training data and predict with it
%
out = y(:,1:opts.gaussdim);
hdg = gaussCreate(nclass,out);
predC = gaussClassifier(hdg,out,realclass,'train',opts.CL);
gaussSave(hdg,opts.gFilename);
fprintf('GAUSS classifier saved in %s\n',opts.gFilename);
%
% how good/bad were another Gaussian classifier based directly on the input
% 'xpre' (bypassing SFA)?
% xpre is either the PCA-dimension-reduced input (if opts.gaussprj=0) or the plain input x.  
xpre = sfa_preproc(hdl,x,opts);
hdx = gaussCreate(nclass,xpre);
predX = gaussClassifier(hdx,xpre,realclass,'train',opts.CL);
if isfield(opts,'xFilename')
    gaussSave(hdx,opts.xFilename);
    fprintf('GAUSS classifier X saved in %s\n',opts.xFilename);
end

errors = numel(find(predC~=realclass));    
errorX = numel(find(predX~=realclass));    
errtrn = [errors errorX]/numel(realclass);
fprintf('                             with SFA   w/o SFA\n');    
fprintf('Error rate on training set = %8.3f%%  %8.3f%%\n', errtrn*100);    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 6-diagram plot
%
if opts.dographics>=1
    % plot the output of the 3 slowest varying functions on training data
    [rc_sort,idx]=sort(realclass);     % sort the vowel data by increasing class number
    figure(1);
    for i=1:3
        subplot(2,3,i+spl); plot(1:trnnum, y(idx,i));
        set(gca, 'PlotBoxAspectRatio', [1,1,1]);
        title(sprintf('output of slow varying function y_%1d(t)',i));
        xlabel('index'); ylabel(sprintf('y_%1d(t)',i));
    end
    spl=spl+3;

    % scatter plot output signal (first 2 dims, all classes in different
    % colors)
    figure(1); 
    spl=spl+1; subplot(2,3,spl);
    hold on; 
    for n=1:nclass
        ind = find(realclass==opts.classes(n));
        plot(y(ind,1),y(ind,2),'Marker','.','MarkerSize',5,'Color',colors(n,:),'LineStyle','none');
        title('the first two output components');
        xlabel('y_1'); ylabel('y_2');
    end 
    set(gca, 'PlotBoxAspectRatio', [1,1,1]); 
    hold off;

    % plot the final class predictor (on training data)
    spl=spl+1; subplot(2,3,spl); 
    hold on;
        predC_sort = predC(idx);
        plot(1:trnnum, predC_sort);
        for n=1:nclass
            ind = find(rc_sort==n);
            plot(ind,predC_sort(ind),'Color',colors(n,:),'LineStyle','-');
        end
        hold off;
        set(gca, 'PlotBoxAspectRatio', [1,1,1]);
        title(sprintf('class predicted by SFA'));
        xlabel('index'); ylabel('predC');
        ylim([0 nclass+1]);
    %print -f1 -dpng -r300 images/class_demo.png
end % dographics>=1

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% some more plots
%
if opts.dographics>=3
    figure(2); clf;
    clc = mod(realclass(idx), 2)*20 - 10;
    r=2; c=2;
    for i=1:min([opts.nclass-1 r*c])
        subplot(r,c,i); hold on;
        %plot(1:trnnum, clc,'Color','red','LineWidth',2,'LineStyle','--');
        fill(1:trnnum, clc, [0.9 0.9 0.9],'LineStyle','none');
        y_i = y(idx,i);
        plot(1:trnnum, y_i);
        set(gca, 'PlotBoxAspectRatio', [1,1,1]);
        set(gca, 'YLim', [min(y_i),max(y_i)]*1.1);
        set(gca, 'XLim', [0, trnnum]);
        title(sprintf('output of slow varying function y_%1d(t)',i));
        xlabel('index'); ylabel(sprintf('y_%1d(t)',i));        
    end
end % dographics>=3

res.errtrn=errtrn;
res.y=y;
res.predC=predC;        % predictions on traing set with SFA + GaussClassifier
res.predX=predX;        % predictions on traing set w/o SFA (only GaussClassifier)
res.hdl=hdl;
res.hdg=hdg;
%
end % sfaClassModel

