 function res = sfaClassPredict(xtst,opts,realc_tst)   
 % sfaClassPredict: use a SFA classification model stored in opts.*Filename,
%                   predict & evaluate on new data (xtst,realc_tst)
%
% see [Berkes05] Pietro Berkes: Pattern recognition with Slow Feature Analysis. 
% Cognitive Sciences EPrint Archive (CogPrint) 4104, http://cogprints.org/4104/ (2005)
%
% INPUT
%   xtst        NTST x IDIM, test input data
%   realc_tst   1 x NTST, test class labels
%   opts        struct with several parameter settings:
%       gaussdim
%       ...
%       *Filename [* = s,g,x] from where to load the models (see sfaClassModel)
%
% OUTPUT
%   res         struct with fields
%       errtst  1 x 2 matrix: error rate with / w/o SFA on test set 
%       ytst    output from SFA when applied to test data
%       predT   predictions with SFA + GaussClassifier on test set
%       predX   predictions w/o SFA (only GaussClassifier) on test set
%               (only if opts.xFilename exists)
%
% Author: Wolfgang Konen, Jan 2011-Mar 2011
%
% See also: sfaClassModel, SFA_EXECUTE, SFA_LOAD, CLASS_DEMO2A, CLASS_DEMO2B 

    fprintf('Loading SFA model from %s\n',opts.sFilename);
    hdl=sfa_load(opts.sFilename);
    fprintf('Loading GAUSS classifier from %s\n',opts.gFilename);
    hdg=gaussLoad(opts.gFilename);
    ytst = sfa_execute(hdl, xtst);
    outtst = ytst(:,1:opts.gaussdim);
    predT = gaussClassifier(hdg,outtst,-1,'apply');
    if isfield(opts,'xFilename')
        fprintf('Loading GAUSS-X classifier from %s\n',opts.xFilename);
        hdx=gaussLoad(opts.xFilename);
        predX = gaussClassifier(hdx,sfa_preproc(hdl,xtst,opts),-1,'apply');
    end
    if exist('realc_tst','var')
        if isfield(opts,'xFilename')
            errorX = numel(find(predX~=realc_tst));    
        else
            errorX = Inf;
        end
        errors = numel(find(predT~=realc_tst));    
        errtst = [errors errorX]/numel(realc_tst);
    else 
        errtst = [Inf Inf];
    end
    fprintf('                             with SFA   w/o SFA\n');    
    fprintf('Error rate on     test set = %8.3f%%  %8.3f%%\n', errtst*100);    
    
    res.errtst=errtst;
    res.ytst=ytst;
    res.predT=predT;        % predictions on test set with SFA + GaussClassifier
    if isfield(opts,'xFilename')
        res.predX=predX;    % predictions on test set w/o SFA (only GaussClassifier)
    end

end % sfaClassPredict