function res = sfa_classify(x,realclass,xtst,realc_tst,opts,pars)
% sfa_classify: classification with SFA
%
% see [Berkes05] Pietro Berkes: Pattern recognition with Slow Feature
% Analysis. Cognitive Sciences EPrint Archive (CogPrint) 4104, http://cogprints.org/4104/ (2005)
%
% INPUT
%   x           NREC x IDIM, training input data
%   xtst        NTST x IDIM, test input data
%   realclass   1 x NREC, training class labels
%   realc_tst   1 x NTST, test class labels
%   opts        struct with several parameter settings
%       idim    number of input dimensions
%       pp_range dims after preproc
%       nclass
%       dographics
%       CL      struct, several settings for the classifier (Gaussian or
%               Nearest Neigbour), see gaussClassifier.m
%
%
% OUTPUT
%   res         struct with fields
%       errtrn  1 x 2 matrix: error rate with / w/o SFA on training set 
%       errtst  1 x 2 matrix: error rate with / w/o SFA on test set 
%       y       (N_REC+N_PB) x IDIM, output from SFA when applied to training data
%       ytst    NTST x IDIM, output from SFA when applied to test data
%       predT   predictions with SFA + GaussClassifier on test set
%       predX   predictions w/o SFA (only GaussClassifier) on test set 
%
% Note that the training set may be augmented by N_PB extra parametric bootstrap
% records, if sfaPBootstrap finds out that N_REC is too small. In this case res.y
% may have more rows than x. 
% If you do not want any parametric bootstrap, set pars.doPB==0.
%
% Author: Wolfgang Konen, May 2009 - Nov 2011
%
% See also: CLASS_DEMO1, CLASS_DEMO2, CLASS_CVDEMO2, CLASS_SWEEP, gaussClassifier

if nargin<6 
    pars = parsDefault();
end
if ~isfield(opts,'gaussprj')
   opts.gaussprj=0;     % 0/1: don't/do skip the PCA-preprocessing step for 'w/o SFA'
end
if ~isfield(opts,'deg')
   opts.deg=2;          % 1/2: create SFA1 or SFA2 object
end
if ~isfield(pars,'doPB')
    pars.doPB=1;        % 1: do parametric bootstrap, if training set too small
end
if ~isfield(opts,'CL')
    opts.CL.algo='gauss';   % use Gaussian classifier
    opts.CL.aligned=1;
    opts.CL.epsD=0.04;
end

nclass  =opts.nclass;

%sfa_clearall;


if opts.dographics>0
    cmap = colormap(jet);
    colors=cmap(mod(15*(1:nclass),size(cmap,1))+1,:); 
    figure(1); clf; set(gcf, 'Position', [89 77 866 638]);
    spl = 0;    % subplot counter
end
if opts.dographics>=2
    % scatter plot input signal (first 2 dims, all classes in different
    % colors)    
    spl=spl+1; subplot(2,3,spl);   
    hold on; 
    for i=1:nclass
        ind = find(realclass==opts.classes(i));
        plot(x(ind,1),x(ind,2),'Marker','.','MarkerSize',5,'Color',colors(i,:),'LineStyle','none');
        xlabel('x_1'); ylabel('x_2');
    end 
    set(gca, 'PlotBoxAspectRatio', [1,1,1]);
    hold off;
end

[realclass,x] = sfaPBootstrap(realclass,x,opts,pars);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Slow Feature Analysis
%
% create a SFA object
if opts.deg==2
    hdl = sfa2_create(opts.pp_range, xp_dim(opts.pp_range), 'PCA2', 'ORD1', 0, opts);
    % perform the preprocessing step
    sfa_step(hdl, x, 'preprocessing');
    % perform the expansion step 
    % IMPORTANT: add patterns one class at a time (!)
    for n=1:nclass
        sfa_step(hdl, x(realclass==opts.classes(n),:), 'expansion','CLASSIF');
    end
    % close the algorithm
    sfa_step(hdl, x, 'sfa','SVD_SFA');        
    %sfa_step(hdl, x, 'sfa','GEN_EIG');      
else % i.e. opts.deg==1
    hdl = sfa1_create(opts.pp_range, 'ORD1', 0);       %--- does not work yet
    % perform the preprocessing step 
    % IMPORTANT: add patterns one class at a time (!)
    for n=1:nclass
        sfa_step(hdl, x(realclass==opts.classes(n),:), 'preprocessing','CLASSIF');
    end
    % close the algorithm
    sfa_step(hdl, x, 'sfa','SVD_SFA');        
    %sfa_step(hdl, x, 'sfa','GEN_EIG');    
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% compute the SFA output signal on the training data
y = sfa_execute(hdl, x);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% train the Gaussian classifier using training data and predict with it
%
out = y(:,1:opts.gaussdim);
hdg = gaussCreate(nclass,out);
predC = gaussClassifier(hdg,out,realclass,'train',opts.CL);
%
% how good/bad were another Gaussian classifier based directly on the input
% 'xpre' (bypassing SFA)?
% xpre is either the PCA-dimension-reduced input (if opts.gaussprj=0) or the plain input x.  
xpre = sfa_preproc(hdl,x,opts);
hdx = gaussCreate(nclass,xpre);
predX = gaussClassifier(hdx,xpre,realclass,'train',opts.CL);

%[confmat,classerr] = mk_confmat(realclass,predC,opts.nclass);
errors = numel(find(predC~=realclass));    
errorX = numel(find(predX~=realclass));    
errtrn = [errors errorX]/numel(realclass);
disp(sprintf('                             with SFA   w/o SFA'));    
disp(sprintf('Error rate on training set = %8.3f%%  %8.3f%%', errtrn*100));
disp(sprintf('training set size = %d', numel(realclass)));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if size(xtst,1)>0
    % if there are test data:
    % compute the SFA output signal on the test data and predict with it
    ytst = sfa_execute(hdl, xtst);
    outtst = ytst(:,1:opts.gaussdim);
    predT = gaussClassifier(hdg,outtst,-1,'apply',opts.CL);
    predX = gaussClassifier(hdx,sfa_preproc(hdl,xtst,opts),-1,'apply',opts.CL);
    errors = numel(find(predT~=realc_tst));    
    errorX = numel(find(predX~=realc_tst));    
    errtst = [errors errorX]/numel(realc_tst);
    fprintf('Error rate on     test set = %8.3f%%  %8.3f%%\n', errtst*100);    
else
    errtst = [];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 6-diagram plot
%
trnnum=numel(realclass);        % number of training cases
tstnum=numel(realc_tst);        % number of test cases
if opts.dographics>=1
    % plot the output of the 3 slowest varying functions on training data
    [rc_sort,idx]=sort(realclass);     % sort the data by increasing class number
    figure(1);
    for i=1:3
        subplot(2,3,i+spl); plot(1:trnnum, y(idx,i));
        set(gca, 'PlotBoxAspectRatio', [1,1,1]);
        title(sprintf('output of slow varying function y_%1d(t)',i));
        xlabel('index'); ylabel(sprintf('y_%1d(t)',i));
    end
    spl=spl+3;

    % scatter plot output signal (first 2 dims, all classes in different
    % colors)
    figure(1); 
    spl=spl+1; subplot(2,3,spl);
    hold on; 
    for n=1:nclass
        ind = find(realclass==opts.classes(n));
        plot(y(ind,1),y(ind,2),'Marker','.','MarkerSize',5,'Color',colors(n,:),'LineStyle','none');
        title('the first two output components');
        xlabel('y_1'); ylabel('y_2');
    end 
    set(gca, 'PlotBoxAspectRatio', [1,1,1]); 
    hold off;

    % plot the final class predictor (on training data)
    spl=spl+1; subplot(2,3,spl); 
    hold on;
        predC_sort = predC(idx);
        plot(1:trnnum, predC_sort);
        for n=1:nclass
            ind = find(rc_sort==n);
            plot(ind,predC_sort(ind),'Color',colors(n,:),'LineStyle','-');
        end
        hold off;
        set(gca, 'PlotBoxAspectRatio', [1,1,1]);
        title(sprintf('class predicted by SFA'));
        xlabel('index'); ylabel('predC');
        ylim([0 nclass+1]);
    %print -f1 -dpng -r300 images/class_demo.png
end % dographics>=1

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% some more plots
%
if opts.dographics>=3
    figure(2); clf;
    clc = mod(realclass(idx), 2)*20 - 10;
    r=2; c=2;
    for i=1:min([opts.nclass-1 r*c])
        subplot(r,c,i); hold on;
        %plot(1:trnnum, clc,'Color','red','LineWidth',2,'LineStyle','--');
        fill(1:trnnum, clc, [0.9 0.9 0.9],'LineStyle','none');
        y_i = y(idx,i);
        plot(1:trnnum, y_i);
        set(gca, 'PlotBoxAspectRatio', [1,1,1]);
        set(gca, 'YLim', [min(y_i),max(y_i)]*1.1);
        set(gca, 'XLim', [0, trnnum]);
        title(sprintf('output of slow varying function y_%1d(t)',i));
        xlabel('index'); ylabel(sprintf('y_%1d(t)',i));        
    end
end % dographics>=3

if opts.dographics>=4
    figure(4); clf;
    r=1; c=2;
    tnum = [trnnum tstnum];
    strg = {'training'; 'test'};
    for i=1:2
        subplot(r,c,i); hold on;
        rc = realclass;
        if (i==2), rc=realc_tst;, end;
        [rc_sort,idx]=sort(rc);     % sort the data by increasing class number
        clc = mod(rc(idx), 2)*20 - 10;
        %plot(1:trnnum, clc,'Color','red','LineWidth',2,'LineStyle','--');
        fill(1:tnum(i), clc, [0.9 0.9 0.9],'LineStyle','none');
        if (i==1), y_i = y(idx,1);, end;
        if (i==2), y_i=ytst(idx,1);, end;
        plot(1:tnum(i), y_i);
        set(gca,'FontSize',18,'Box','on');
        set(gca, 'PlotBoxAspectRatio', [1,1,1]);
        set(gca, 'YLim', [min(y_i),max(y_i)]*1.1);
        set(gca, 'XLim', [0, tnum(i)]);
        title(sprintf(' y_1(t) on %s set',strg{i}));
        xlabel('index'); ylabel('y_1(t)');     
        
        DO_PRINT=1;
        if (DO_PRINT==1)
            epsdir = 'images/eps/';
            if (~exist(epsdir)) mkdir(epsdir); end; 
            filename=[epsdir 'y1_train_test.eps'];
            print('-f4','-depsc2','-r100',filename);
            disp(sprintf('Figure 4 saved to %s\n',filename));
        end
    end
end % dographics>=4

res.errtrn=errtrn;
res.errtst=errtst;
res.y=y;                % (N_REC+N_PB) x IDIM, output data of sfa_execute on training set
res.ytst=ytst;          % NTST x IDIM, output data of sfa_execute on test set 
res.predT=predT;        % predictions on test set with SFA + GaussClassifier
res.predX=predX;        % predictions on test set w/o SFA (only GaussClassifier)
%
end % sfa_classify

